package hamster;

/**
 * Die Klasse erweitert den Befehlssatz eines normalen Hamsters um viele
 * nuetzliche Befehle.
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (25.01.2006)
 * 
 */
public class AllroundHamster extends Hamster {

	/**
	 * initialisiert einen neuen AllroundHamster mit den uebergebenen Werten
	 * 
	 * @param reihe
	 *            Reihe der Kachel
	 * @param spalte
	 *            Spalte der Kachel
	 * @param blickrichtung
	 *            Blickrichtung des Hamsters
	 * @param anzahlKoerner
	 *            Anzahl an Koernern im Maul des Hamsters
	 */
	public AllroundHamster(int reihe, int spalte, int blickrichtung,
			int anzahlKoerner) {
		super(reihe, spalte, blickrichtung, anzahlKoerner);
	}

	/**
	 * laedt ein vordefiniertes Territorium und erzeugt und initialisiert darin
	 * einen neuen AllroundHamster mit den uebergebenen Werten
	 * 
	 * @param reihe
	 *            Reihe der Kachel
	 * @param spalte
	 *            Spalte der Kachel
	 * @param blickrichtung
	 *            Blickrichtung des Hamsters
	 * @param anzahlKoerner
	 *            Anzahl an Koernern im Maul des Hamsters
	 * @param territoriumsDatei
	 *            Name einer Datei mit einem zu ladenden Territorium
	 */
	public AllroundHamster(int reihe, int spalte, int blickrichtung,
			int anzahlKoerner, String territoriumsDatei) {
		super();
		Territorium.ladeTerritorium(territoriumsDatei);
		this.init(reihe, spalte, blickrichtung, anzahlKoerner);
	}

	/**
	 * initialisiert einen neuen AllroundHamster mit den Attributwerten eines
	 * bereits existierenden Hamsters
	 * 
	 * @param existierenderHamster
	 *            existierender Hamster
	 */
	public AllroundHamster(Hamster existierenderHamster) {
		super(existierenderHamster);
	}

	/**
	 * laedt ein vordefiniertes Territorium und initialisiert einen neuen
	 * AllroundHamster mit den Attributwerten des Standard-Hamsters
	 * 
	 * @param territoriumsDatei
	 *            Name einer Datei mit einem zu ladenden Territorium
	 */
	public AllroundHamster(String territoriumsDatei) {
		super();
		Territorium.ladeTerritorium(territoriumsDatei);
		Hamster ham = Hamster.getStandardHamster();
		this.init(ham.getReihe(), ham.getSpalte(), ham.getBlickrichtung(), ham
				.getAnzahlKoerner());
	}

	/**
	 * Konstruktor zum Erzeugen eines nicht initialisierten AllroundHamsters
	 */
	public AllroundHamster() {
		super();
	}

	/**
	 * der Hamster dreht sich um 180 Grad
	 */
	public void kehrt() {
		this.linksUm();
		this.linksUm();
	}

	/**
	 * der Hamster dreht sich um 90 Grad nach nach rechts
	 */
	public void rechtsUm() {
		this.kehrt();
		this.linksUm();
	}

	/**
	 * der Hamster laeuft "anzahl" Schritte, maximal jedoch bis zur naechsten
	 * Mauer; geliefert wird die tatsaechliche Anzahl gelaufener Schritte
	 * 
	 * @param anzahl
	 *            maximale Anzahl an zu laufenden Schritten
	 * @return die tatsaechliche Anzahl gelaufener Schritte
	 */
	public int vor(int anzahl) {
		int schritte = 0;
		while (this.vornFrei() && anzahl > 0) {
			this.vor();
			schritte = schritte + 1;
			anzahl = anzahl - 1;
		}
		return schritte;
	}

	/**
	 * der Hamster legt "anzahl" Koerner ab, maximal jedoch so viele, wie er im
	 * Maul hat; geliefert wird die tatsaechliche Anzahl abgelegter Koerner
	 * 
	 * @param anzahl
	 *            maximale Anzahl an abzulegenden Koernern
	 * @return die tatsaechliche Anzahl abgelegter Koerner
	 */
	public int gib(int anzahl) {
		int abgelegteKoerner = 0;
		while (!this.maulLeer() && anzahl > 0) {
			this.gib();
			abgelegteKoerner = abgelegteKoerner + 1;
			anzahl = anzahl - 1;
		}
		return abgelegteKoerner;
	}

	/**
	 * der Hamster frisst "anzahl" Koerner, maximal jedoch so viele, wie auf der
	 * aktuellen Kachel liegen; geliefert wird die tatsaechliche Anzahl
	 * gefressener Koerner
	 * 
	 * @param anzahl
	 *            maximale Anzahl an zu fressenden Koernern
	 * @return die tatsaechliche Anzahl gefressener Koerner
	 */
	public int nimm(int anzahl) {
		int gefresseneKoerner = 0;
		while (this.kornDa() && anzahl > 0) {
			this.nimm();
			gefresseneKoerner = gefresseneKoerner + 1;
			anzahl = anzahl - 1;
		}
		return gefresseneKoerner;
	}

	/**
	 * der Hamster legt alle Koerner, die er im Maul hat, auf der aktuellen
	 * Kachel ab; geliefert wird die Anzahl abgelegter Koerner
	 * 
	 * @return Anzahl abgelegter Koerner
	 */
	public int gibAlle() {
		int abgelegteKoerner = 0;
		while (!this.maulLeer()) {
			this.gib();
			abgelegteKoerner = abgelegteKoerner + 1;
		}
		return abgelegteKoerner;
	}

	/**
	 * der Hamster frisst alle Koerner auf der aktuellen Kachel; geliefert wird
	 * die Anzahl gefressener Koerner
	 * 
	 * @return Anzahl gefressener Koerner
	 */
	public int nimmAlle() {
		int gefresseneKoerner = 0;
		while (this.kornDa()) {
			this.nimm();
			gefresseneKoerner = gefresseneKoerner + 1;
		}
		return gefresseneKoerner;
	}

	/**
	 * der Hamster laeuft bis zur naechsten Mauer; geliefert wird die Anzahl
	 * ausgefuehrter Schritte
	 * 
	 * @return Anzahl ausgefuehrter Schritte
	 */
	public int laufeZurWand() {
		int schritte = 0;
		while (this.vornFrei()) {
			this.vor();
			schritte = schritte + 1;
		}
		return schritte;
	}

	/**
	 * der Hamster testet, ob links von ihm die Kachel frei ist
	 * 
	 * @return true, falls die Kachel links von dem Hamster frei ist; sonst
	 *         false
	 */
	public boolean linksFrei() {
		this.linksUm();
		boolean frei = this.vornFrei();
		this.rechtsUm();
		return frei;
	}

	/**
	 * der Hamster testet, ob rechts von ihm die Kachel frei ist
	 * 
	 * @return true, falls die Kachel rechts von dem Hamster frei ist; sonst
	 *         false
	 */
	public boolean rechtsFrei() {
		this.rechtsUm();
		boolean frei = this.vornFrei();
		this.linksUm();
		return frei;
	}

	/**
	 * der Hamster testet, ob hinter ihm die Kachel frei ist
	 * 
	 * @return true, falls die Kachel hinter dem Hamster frei ist; sonst false
	 */
	public boolean hintenFrei() {
		this.kehrt();
		boolean frei = this.vornFrei();
		this.kehrt();
		return frei;
	}

	/**
	 * der Hamster dreht sich so lange um, bis er in die uebergebene
	 * Blickrichtung schaut
	 * 
	 * @param richtung
	 *            die Richtung, in die der Hamster schauen soll
	 */
	public void setzeBlickrichtung(int richtung) {
		while (this.getBlickrichtung() != richtung) {
			this.linksUm();
		}
	}

	/**
	 * der Hamster laeuft in der Spalte, in der er gerade steht, zur angegebenen
	 * Reihe; Voraussetzung: die Reihe existiert und es befinden sich keine
	 * Mauern im Territorium bzw. auf dem gewaehlten Weg
	 * 
	 * @param reihe
	 *            die Reihe, zu der der Hamster laufen soll
	 */
	public void laufeZuReihe(int reihe) {
		if (reihe == this.getReihe())
			return;
		if (reihe > this.getReihe()) {
			this.setzeBlickrichtung(Hamster.SUED);
		} else {
			this.setzeBlickrichtung(Hamster.NORD);
		}
		while (reihe != this.getReihe()) {
			this.vor();
		}
	}

	/**
	 * der Hamster laeuft in der Reihe, in der er gerade steht, zur angegebenen
	 * Spalte; Voraussetzung: die Spalte existiert und es befinden sich keine
	 * Mauern im Territorium bzw. auf dem gewaehlten Weg
	 * 
	 * @param spalte
	 *            die Spalte, zu der der Hamster laufen soll
	 */
	public void laufeZuSpalte(int spalte) {
		if (spalte == this.getSpalte())
			return;
		if (spalte > this.getSpalte()) {
			this.setzeBlickrichtung(Hamster.OST);
		} else {
			this.setzeBlickrichtung(Hamster.WEST);
		}
		while (spalte != this.getSpalte()) {
			this.vor();
		}
	}

	/**
	 * der Hamster laeuft zur Kachel (reihe/spalte); Voraussetzung: die Kachel
	 * existiert und es befinden sich keine Mauern im Territorium bzw. auf dem
	 * gewaehlten Weg
	 * 
	 * @param reihe
	 *            die Reihe, zu der der Hamster laufen soll
	 * @param spalte
	 *            die Spalte, zu der der Hamster laufen soll
	 */
	public void laufeZuKachel(int reihe, int spalte) {
		laufeZuReihe(reihe);
		laufeZuSpalte(spalte);
	}
}
